@@ -16,11 +16,14 @@
 #include <ngx_core.h>
 #include <ngx_http.h>
 #include <fmp4/mp4_process.h>
+#include <fmp4/api_process.h>
 #include <fmp4/mp4_rewrite.h>
 #include <fmp4/mp4_options.h>
 #include <fmp4/moov.h>
 #include <fmp4/output_bucket.h>
 #include <fmp4/post_handler.h>
+#include <fmp4/headers_t.h>
+#include <fmp4/buckets_t.h>
 
 #if 0
 /* Mod-H264-Streaming configuration
@@ -72,6 +75,8 @@ typedef struct
   ngx_str_t s3_access_key;
   ngx_str_t s3_region;
 
+  ngx_flag_t usp_content_id;
+
 } ngx_http_streaming_loc_conf_t;
 
 typedef struct
@@ -82,18 +87,24 @@ typedef struct
 
 typedef struct
 {
-  mp4_global_context_t* gctx;
+  mp4_global_context_t const* gctx;
   ngx_str_t license;
 
 } ngx_http_streaming_main_conf_t;
 
 static char* ngx_streaming(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
 static ngx_int_t ngx_streaming_handler_get(ngx_http_request_t *r,
-                                           mp4_process_context_t* context);
+                                           mp4_process_context_t* context,
+                                           fmp4_http_method_t http_method,
+                                           int is_rest_api,
+                                           char const* filename);
 #if defined(BUILDING_SMOOTH_STREAMING)
 static ngx_int_t ngx_http_streaming_init(ngx_conf_t* cf);
 static ngx_int_t ngx_streaming_handler_post(ngx_http_request_t *r,
-                                            mp4_process_context_t* context);
+                                            mp4_process_context_t* context,
+                                            fmp4_http_method_t http_method,
+                                            int is_rest_api,
+                                            char const* filename);
 static char* ngx_ism_proxy_pass(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
 static char* ngx_usp_prefer_static(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
 static char* ngx_usp_iss_pass_through(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
@@ -106,6 +117,7 @@ static char* ngx_usp_disable_mmap(ngx_conf_t* cf, ngx_command_t* cmd, void* conf
 static char* ngx_usp_s3_secret_key(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
 static char* ngx_usp_s3_access_key(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
 static char* ngx_usp_s3_region(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
+static char* ngx_usp_content_id(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
 
 #endif
 static char* ngx_usp_license_key(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
@@ -260,6 +272,14 @@ static ngx_command_t ngx_streaming_commands[] =
     NULL
   },
 
+  { ngx_string("usp_content_id"),
+    NGX_HTTP_LOC_CONF|NGX_CONF_NOARGS,
+    ngx_usp_content_id,
+    NGX_HTTP_LOC_CONF_OFFSET,
+    0,
+    NULL
+  },
+
   ngx_null_command
 };
 #endif
@@ -341,6 +361,7 @@ ngx_http_streaming_create_loc_conf(ngx_conf_t *cf)
   conf->s3_access_key.len = NGX_CONF_UNSET;
   conf->s3_region.data = NULL;
   conf->s3_region.len = NGX_CONF_UNSET;
+  conf->usp_content_id = NGX_CONF_UNSET;
 
   return conf;
 }
@@ -365,6 +386,8 @@ ngx_http_streaming_merge_loc_conf(ngx_conf_t *cf, void *parent, void *child)
   ngx_conf_merge_str_value(conf->s3_access_key, prev->s3_access_key, 0);
   ngx_conf_merge_str_value(conf->s3_region, prev->s3_region, 0);
 
+  ngx_conf_merge_value(conf->usp_content_id, prev->usp_content_id, 0);
+
   return NGX_CONF_OK;
 }
 
@@ -517,7 +540,7 @@ static ngx_int_t ngx_streaming_post_rewrite_handler(ngx_http_request_t *r)
 #if defined(BUILDING_SMOOTH_STREAMING)
 static void ngx_http_cleanup_mp4_global_context(void* data)
 {
-  mp4_global_context_t* gctx = data;
+  mp4_global_context_t const* gctx = data;
   libfmp4_global_exit(gctx);
 }
 
@@ -674,7 +697,8 @@ log_error_callback(void* context, fmp4_log_level_t level, char const* str)
 }
 
 static ngx_int_t
-ngx_streaming_handler_paths(ngx_http_request_t *r, mp4_process_context_t* ctx)
+ngx_streaming_handler_paths(ngx_http_request_t *r, mp4_process_context_t* ctx,
+                            char** filename)
 {
   u_char                      *last;
   size_t                      root;
@@ -687,8 +711,8 @@ ngx_streaming_handler_paths(ngx_http_request_t *r, mp4_process_context_t* ctx)
 
   slcf = ngx_http_get_module_loc_conf(r, ngx_http_streaming_module);
 
-  ctx->prefer_static_ =
-    slcf->usp_prefer_static == NGX_CONF_UNSET ? 0 : slcf->usp_prefer_static;
+  mp4_process_context_set_prefer_static(ctx,
+    slcf->usp_prefer_static == NGX_CONF_UNSET ? 0 : slcf->usp_prefer_static);
 
   if(slcf->url.data)
   {
@@ -728,14 +752,6 @@ ngx_streaming_handler_paths(ngx_http_request_t *r, mp4_process_context_t* ctx)
     ngx_open_file_info_t of;
     http_status = open_path(r, clcf, &path, &of);
 
-#if 0 // don't rely on (stale) file metadata
-    if(http_status == NGX_OK)
-    {
-      ctx->filesize_ = of.size;
-      r->headers_out.last_modified_time = of.mtime;
-    }
-#endif
-
     // a file-not-found may occur when ingesting a live stream that has no server manifest file.
     if(http_status != NGX_OK && http_status != NGX_HTTP_NOT_FOUND)
     {
@@ -743,8 +759,8 @@ ngx_streaming_handler_paths(ngx_http_request_t *r, mp4_process_context_t* ctx)
     }
   }
 
-  ctx->filename_ = ngx_pcalloc(r->pool, path.len+1);
-  ngx_memcpy((void*)ctx->filename_, path.data, path.len);
+  *filename = ngx_pcalloc(r->pool, path.len+1);
+  ngx_memcpy(*filename, path.data, path.len);
 
   ngx_log_debug(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
                "ngx_streaming_handler_paths: exit");
@@ -810,80 +826,81 @@ static ngx_int_t ngx_streaming_handler(ngx_http_request_t *r)
 
   smcf = ngx_http_get_module_main_conf(r, ngx_http_streaming_module);
 
-  mp4_process_context_t* context = ngx_palloc(r->pool, sizeof(mp4_process_context_t));
-  mp4_process_context_init(context, smcf->gctx);
+  mp4_process_context_t* context = mp4_process_context_init(smcf->gctx);
   ngx_http_add_cleanup(r, ngx_http_cleanup_mp4_process_context, context);
 
-  context->get_server_variable_callback_ = get_server_variable;
-  context->get_server_variable_context_ = r;
-  context->log_error_callback_ = log_error_callback;
-  context->log_error_context_ = r;
-  context->verbose_ = nxg_log_level_to_fmp4_log_level(r);
-  context->method_ = ngx_get_method(r);
+  mp4_process_context_set_variable_callback(context, get_server_variable, r);
+  mp4_process_context_set_log_error_callback(context, log_error_callback, r);
+  mp4_process_context_set_verbose(context, nxg_log_level_to_fmp4_log_level(r));
 
-  context->prefer_static_ = slcf->usp_prefer_static == NGX_CONF_UNSET ? 0
-                          : slcf->usp_prefer_static;
-  context->iss_pass_through_ = slcf->usp_iss_pass_through == NGX_CONF_UNSET ? 0
-                             : slcf->usp_iss_pass_through;
-  context->dynamic_time_shift_buffer_depth_ =
+  mp4_process_context_set_prefer_static(context,
+    slcf->usp_prefer_static == NGX_CONF_UNSET ? 0 :
+    slcf->usp_prefer_static);
+  mp4_process_context_set_iss_pass_through(context,
+    slcf->usp_iss_pass_through == NGX_CONF_UNSET ? 0 :
+    slcf->usp_iss_pass_through);
+  mp4_process_context_set_dynamic_time_shift_buffer_depth(context,
     slcf->usp_dynamic_time_shift_buffer_depth == NGX_CONF_UNSET ? 0 :
-    slcf->usp_dynamic_time_shift_buffer_depth;
+    slcf->usp_dynamic_time_shift_buffer_depth);
 
   if(slcf->usp_disable_mmap)
   {
-    context->usp_enable_mmap_ = 0;
+    mp4_process_context_set_enable_mmap(context, 0);
   }
 
   if(slcf->s3_secret_key.data && slcf->s3_access_key.data)
   {
-    context->s3_secret_key_ = (char const*)slcf->s3_secret_key.data;
-    context->s3_access_key_ = (char const*)slcf->s3_access_key.data;
-    if(slcf->s3_region.data)
-    {
-      context->s3_region_ = (char const*)slcf->s3_region.data;
-    }
+    mp4_process_context_set_s3_parameters(context,
+      (char const*)slcf->s3_secret_key.data,
+      (char const*)slcf->s3_access_key.data,
+      (char const*)slcf->s3_region.data,
+      NULL, 0);
+    // NOTE: no security_token, nor use_headers support at this time
   }
-  ngx_int_t http_status = ngx_streaming_handler_paths(r, context);
+
+  mp4_process_context_set_content_id(context,
+    slcf->usp_content_id == NGX_CONF_UNSET ? 0 : slcf->usp_content_id);
+
+  char* filename = NULL;
+  ngx_int_t http_status = ngx_streaming_handler_paths(r, context, &filename);
   if(NGX_OK != http_status)
   {
-    //mp4_process_context_exit(context);
     return http_status;
   }
 
-  mp4_split_options_t* options = context->options_;
+  mp4_options_t* options = mp4_process_context_get_options(context);
   if(r->args.len)
   {
-    if(!mp4_split_options_set(options,
-        (const char*)r->args.data, r->args.len))
+    if(!mp4_options_set(options, (const char*)r->args.data, r->args.len))
     {
-      //mp4_process_context_exit(context);
       return NGX_HTTP_BAD_REQUEST;
     }
   }
 
-  if(slcf->usp_handle_api > 0)
-  {
-    context->is_rest_api_ = 1;
-  }
+  int is_rest_api = slcf->usp_handle_api > 0;
 
   if(slcf->usp_handle_smpd > 0)
   {
-    context->is_storage_mpd_ = 1;
+    mp4_process_context_set_is_storage_mpd(context, 1);
   }
 
+  fmp4_http_method_t http_method = ngx_get_method(r);
+
   if(r->method & (NGX_HTTP_GET|NGX_HTTP_HEAD))
   {
-    http_status = ngx_streaming_handler_get(r, context);
+    http_status = ngx_streaming_handler_get(r, context, http_method,
+      is_rest_api, filename);
   }
 #if defined(BUILDING_SMOOTH_STREAMING)
   else if(r->method & (NGX_HTTP_POST|NGX_HTTP_PUT))
   {
-    http_status = ngx_streaming_handler_post(r, context);
+    http_status = ngx_streaming_handler_post(r, context, http_method,
+      is_rest_api, filename);
   }
 
-  if(r->method & NGX_HTTP_DELETE && context->is_rest_api_)
+  if(r->method & NGX_HTTP_DELETE && is_rest_api)
   {
-    http_status = mp4_process(context);
+    http_status = mp4_api_process(context, http_method, filename);
     r->headers_out.content_length_n = 0;
     r->headers_out.status = http_status;
     r->header_only = 1;
@@ -891,8 +908,6 @@ static ngx_int_t ngx_streaming_handler(ngx_http_request_t *r)
   }
 #endif
 
-  //mp4_process_context_exit(context);
-
   // Note: do not do any logging here to r->connection->log. The connection may
   // already have been finalized.
 //ngx_log_debug(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
@@ -1018,20 +1033,22 @@ ngx_http_fmp4_request_body_filter(ngx_http_request_t* r,
 }
 
 static ngx_int_t
-ngx_streaming_handler_post(ngx_http_request_t *r, mp4_process_context_t* ctx)
+ngx_streaming_handler_post(ngx_http_request_t *r, mp4_process_context_t* ctx,
+                           fmp4_http_method_t http_method, int is_rest_api,
+                           char const* filename)
 {
   ngx_int_t http_status;
 
-  if(!mp4_ends_with(ctx->filename_, ".isml"))
+  if(!mp4_ends_with(filename, ".isml"))
   {
     return NGX_HTTP_FORBIDDEN;
   }
 
-  mp4_split_options_t const* options = ctx->options_;
-  char const* file = mp4_split_options_get_file(options);
+  mp4_options_t const* options = mp4_process_context_get_options(ctx);
+  char const* file = mp4_options_get_file(options);
 
   // Connect
-  if(!file[0] && !ctx->is_rest_api_)
+  if(!file[0] && !is_rest_api)
   {
     r->headers_out.content_length_n = 0;
     r->headers_out.status = NGX_HTTP_OK;
@@ -1041,23 +1058,24 @@ ngx_streaming_handler_post(ngx_http_request_t *r, mp4_process_context_t* ctx)
   }
 
   // Pusing to .isml/Streams(...) disable api_context
-  if (file[0] && ctx->is_rest_api_)
+  if (file[0] && is_rest_api)
   {
-    ctx->is_rest_api_ = 0;
+    is_rest_api = 0;
   }
 
   if(!strcmp(file, "purge"))
   {
-    fmp4_result result = mp4_process(ctx);
+    fmp4_result result = mp4_process(ctx, http_method, filename);
 
     if(result != NGX_HTTP_OK)
     {
+      char const* result_text = mp4_process_context_get_result_text(ctx);
       ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
                     "mp4_process(%s/%s) returned: %s %s",
-                    ctx->filename_,
+                    filename,
                     file,
-                    fmp4_result_to_string(ctx->result_),
-                    ctx->result_text_);
+                    fmp4_result_to_string(result),
+                    result_text);
 
       return result;
     }
@@ -1072,14 +1090,17 @@ ngx_streaming_handler_post(ngx_http_request_t *r, mp4_process_context_t* ctx)
 
   ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
                 "ngx_streaming_handler_post: manifest=%s",
-                (char const*)ctx->filename_);
+                filename);
 
-  post_handler_t* post_handler = create_post_handler(ctx);
+  post_handler_t* post_handler = is_rest_api ?
+    create_post_handler_api(ctx, http_method, filename) :
+      create_post_handler(ctx, filename);
 
   if(post_handler == NULL)
   {
+    char const* result_text = mp4_process_context_get_result_text(ctx);
     ngx_log_debug(NGX_LOG_ERR, r->connection->log, 0,
-                  "usp_post_handler returned: %s", ctx->result_text_);
+                  "usp_post_handler returned: %s", result_text);
 
     return NGX_HTTP_INTERNAL_SERVER_ERROR;
   }
@@ -1122,7 +1143,10 @@ ngx_streaming_handler_post(ngx_http_request_t *r, mp4_process_context_t* ctx)
 
 static ngx_int_t
 ngx_streaming_handler_get(ngx_http_request_t *r,
-                          mp4_process_context_t* context)
+                          mp4_process_context_t* context,
+                          fmp4_http_method_t http_method,
+                          int is_rest_api,
+                          char const* filename)
 {
   ngx_int_t                   http_status;
   ngx_log_t                   *log;
@@ -1137,19 +1161,18 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
 
   clcf = ngx_http_get_module_loc_conf(r, ngx_http_core_module);
 
-  mp4_split_options_t* options = context->options_;
-  char const* file = mp4_split_options_get_file(options);
+  mp4_options_t* options = mp4_process_context_get_options(context);
+  char const* file = mp4_options_get_file(options);
 
   ngx_log_debug(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
-                "ngx_streaming_handler_get: %s", (char const*)context->filename_);
+                "ngx_streaming_handler_get: %s", filename);
 
   // If we are requesting the server manifest file, then it may an encoding tool
   // that is using the GET request to get the authorization setup for the POST.
-  if(mp4_ends_with((char const*)context->filename_, ".isml"))
+  if(mp4_ends_with(filename, ".isml"))
   {
     if(mp4_starts_with(file, "Streams("))
     {
-      //mp4_process_context_exit(context);
       r->headers_out.content_length_n = 0;
       r->headers_out.status = NGX_HTTP_OK;
       r->header_only = 1;
@@ -1160,7 +1183,6 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
 //    // we are either serving dynamically generated files, or fragments
 //    if(!file[0])
 //    {
-//      mp4_process_context_exit(&context);
 //      return NGX_HTTP_FORBIDDEN;
 //    }
   }
@@ -1171,12 +1193,15 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
   r->root_tested = !r->error_page;
 
   {
-    r->headers_out.status = http_status = mp4_process(context);
+    r->headers_out.status = http_status = is_rest_api ?
+      mp4_api_process(context, http_method, filename) :
+        mp4_process(context, http_method, filename);
 
-    headers_t* headers = context->headers_;
-    buckets_t* buckets = context->buckets_;
+    headers_t* headers = mp4_process_context_get_headers(context);
+    buckets_t* buckets = mp4_process_context_get_buckets(context);
 
-    if(headers->x_usp_header1[0])
+    char const* x_usp_header1 = headers_get_x_usp_header1(headers);
+    if(x_usp_header1 && x_usp_header1[0])
     {
       ngx_table_elt_t* h = ngx_list_push(&r->headers_out.headers);
       if(h == NULL)
@@ -1187,11 +1212,12 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
 
       h->key.len = sizeof("X-USP-Info1") - 1;
       h->key.data = (u_char*)"X-USP-Info1";
-      h->value.len = strlen(headers->x_usp_header1);
-      h->value.data = (u_char*)headers->x_usp_header1;
+      h->value.len = strlen(x_usp_header1);
+      h->value.data = (u_char*)x_usp_header1;
     }
 
-    if(headers->x_usp_header2[0])
+    char const* x_usp_header2 = headers_get_x_usp_header2(headers);
+    if(x_usp_header2 && x_usp_header2[0])
     {
       ngx_table_elt_t* h = ngx_list_push(&r->headers_out.headers);
       if(h == NULL)
@@ -1202,23 +1228,25 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
 
       h->key.len = sizeof("X-USP-Info2") - 1;
       h->key.data = (u_char*)"X-USP-Info2";
-      h->value.len = strlen(headers->x_usp_header2);
-      h->value.data = (u_char*)headers->x_usp_header2;
+      h->value.len = strlen(x_usp_header2);
+      h->value.data = (u_char*)x_usp_header2;
     }
 
     // Add Last-Modified
-    if(headers->updated_at_)
+    uint64_t updated_at = headers_get_updated_at(headers);
+    if(updated_at)
     {
-      r->headers_out.last_modified_time = headers->updated_at_ / 1000000;
+      r->headers_out.last_modified_time = updated_at / 1000000;
     }
 
     // compute expiry with seconds accuracy for interoperable results with nginx
     typedef uint64_t seconds_t;
 
     // Add X-USP-Last-Modified
-    if(headers->updated_at_experimental_)
+    uint64_t updated_at_experimental = headers_get_updated_at_experimental(headers);
+    if(updated_at_experimental)
     {
-      seconds_t updated_time = headers->updated_at_experimental_ / 1000000;
+      seconds_t updated_time = updated_at_experimental / 1000000;
       ngx_table_elt_t* h = ngx_list_push(&r->headers_out.headers);
       if(h == NULL)
       {
@@ -1242,14 +1270,16 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
       ngx_str_t cache_control = ngx_null_string;
       ngx_str_t expires = ngx_null_string;
 
-      if(headers->cache_control_)
+      char const* cache_control_hdr = headers_get_cache_control(headers);
+      uint64_t expires_at = headers_get_expires_at(headers);
+      if(cache_control_hdr && cache_control_hdr[0])
       {
-        cache_control.len = strlen(headers->cache_control_);
-        cache_control.data = (u_char*)headers->cache_control_;
+        cache_control.len = strlen(cache_control_hdr);
+        cache_control.data = (u_char*)cache_control_hdr;
       }
-      else if(headers->expires_at_)
+      else if(expires_at)
       {
-        seconds_t expires_time = headers->expires_at_ / 1000000;
+        seconds_t expires_time = expires_at / 1000000;
         seconds_t request_time = r->start_sec;
 
         // use expiry from backend only if it's at least a second ahead of request
@@ -1368,9 +1398,10 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
     }
 
     // Add Sunset header
-    if(headers->sunset_at_)
+    uint64_t sunset_at = headers_get_sunset_at(headers);
+    if(sunset_at)
     {
-      seconds_t sunset_time = headers->sunset_at_ / 1000000;
+      seconds_t sunset_time = sunset_at / 1000000;
 
       ngx_table_elt_t* h = ngx_list_push(&r->headers_out.headers);
       if(h == NULL)
@@ -1390,21 +1421,38 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
       ngx_http_time(h->value.data, sunset_time);
     }
 
-    // Add the link header for start/next
-    if(headers->link_[0])
+    // Add the Link header for start/next
+    char const* link = headers_get_link(headers);
+    if(link && link[0])
     {
       ngx_table_elt_t* h = ngx_list_push(&r->headers_out.headers);
       if(h == NULL)
       {
-        //mp4_process_context_exit(&context);
         return NGX_HTTP_INTERNAL_SERVER_ERROR;
       }
       h->hash = 1;
 
       h->key.len = sizeof("Link") - 1;
       h->key.data = (u_char*)"Link";
-      h->value.len = strlen(headers->link_);
-      h->value.data = (u_char*)headers->link_;
+      h->value.len = strlen(link);
+      h->value.data = (u_char*)link;
+    }
+
+    // Add the X-USP-Content-ID header for start/next
+    char const* content_id = headers_get_content_id(headers);
+    if(content_id && content_id[0])
+    {
+      ngx_table_elt_t* h = ngx_list_push(&r->headers_out.headers);
+      if(h == NULL)
+      {
+        return NGX_HTTP_INTERNAL_SERVER_ERROR;
+      }
+      h->hash = 1;
+
+      h->key.len = sizeof("X-USP-Content-ID") - 1;
+      h->key.data = (u_char*)"X-USP-Content-ID";
+      h->value.len = strlen(content_id);
+      h->value.data = (u_char*)content_id;
     }
 
     // Set the content length
@@ -1427,7 +1475,8 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
     }
 
     // Add ETag header
-    if(headers->etag_[0])
+    char const* etag_hdr = headers_get_etag(headers);
+    if(etag_hdr && etag_hdr[0])
     {
       ngx_table_elt_t* etag = ngx_list_push(&r->headers_out.headers);
       if(etag == NULL)
@@ -1438,8 +1487,8 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
       etag->hash = 1;
       ngx_str_set(&etag->key, "ETag");
 
-      etag->value.len = strlen(headers->etag_);
-      etag->value.data = (u_char*)headers->etag_;
+      etag->value.len = strlen(etag_hdr);
+      etag->value.data = (u_char*)etag_hdr;
 
       r->headers_out.etag = etag;
     }
@@ -1453,18 +1502,21 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
 
     if(r->headers_out.status != 200)
     {
+      fmp4_result result = mp4_process_context_get_result(context);
+      char const* result_text = mp4_process_context_get_result_text(context);
       ngx_log_error(NGX_LOG_ERR, log, 0,
                     "mp4_process(%s/%s) returned: %s %s",
-                    context->filename_,
+                    filename,
                     file,
-                    fmp4_result_to_string(context->result_),
-                    context->result_text_);
+                    fmp4_result_to_string(result),
+                    result_text);
     }
     else
     {
       // Set the content type
-      r->headers_out.content_type.len = strlen(headers->content_type_);
-      r->headers_out.content_type.data = (u_char*)headers->content_type_;
+      char const* content_type = headers_get_content_type(headers);
+      r->headers_out.content_type.len = strlen(content_type);
+      r->headers_out.content_type.data = (u_char*)content_type;
     }
 
     // Nginx properly handles range requests, even when we're sending chunks
@@ -1500,7 +1552,6 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
           *chain = ngx_pcalloc(r->pool, sizeof(ngx_chain_t));
           if(b == NULL || *chain == NULL)
           {
-            //mp4_process_context_exit(&context);
             return NGX_HTTP_INTERNAL_SERVER_ERROR;
           }
 
@@ -1520,13 +1571,12 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
             b->file = ngx_pcalloc(r->pool, sizeof(ngx_file_t));
             if(b->file == NULL)
             {
-              //mp4_process_context_exit(&context);
               return NGX_HTTP_INTERNAL_SERVER_ERROR;
             }
 
             // poor man's caching
             if(filename_prev == NULL ||
-               fname != filename_prev && strcmp(fname, filename_prev) != NULL)
+               (fname != filename_prev && strcmp(fname, filename_prev) != 0))
             {
               //ngx_log_error(NGX_LOG_INFO, log, 0, "poor man's cache miss on %s", fname);
               filename_prev = fname;
@@ -1546,7 +1596,6 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
 
               if(http_status != NGX_OK)
               {
-                //mp4_process_context_exit(&context);
                 return http_status;
               }
             }
@@ -1576,7 +1625,6 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
             b->pos = ngx_pcalloc(r->pool, size);
             if(b->pos == NULL)
             {
-              //mp4_process_context_exit(&context);
               return NGX_HTTP_INTERNAL_SERVER_ERROR;
             }
             b->last = b->pos + size;
@@ -1592,7 +1640,6 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
           (*chain)->next = NULL;
           chain = &(*chain)->next;
         }
-        //mp4_process_context_exit(&context);
       }
 
       return ngx_http_output_filter(r, out);
@@ -1745,6 +1792,15 @@ static char* ngx_usp_s3_region(ngx_conf_t* cf, ngx_command_t* cmd, void* conf)
   return NGX_CONF_OK;
 }
 
+static char* ngx_usp_content_id(ngx_conf_t* cf, ngx_command_t* cmd, void* conf)
+{
+  ngx_http_streaming_loc_conf_t* slcf = conf;
+
+  slcf->usp_content_id = 1;
+
+  return NGX_CONF_OK;
+}
+
 #endif
 
 static char*
@@ -1797,7 +1853,7 @@ static ngx_int_t ngx_http_add_cleanup(ngx_http_request_t* r,
 
 static void ngx_http_cleanup_mp4_process_context(void* context)
 {
-  mp4_process_context_exit((mp4_process_context_t*)context);
+  mp4_process_context_exit((mp4_process_context_t const*)context);
 }
 
 // End Of File
